@@ -22,6 +22,7 @@ import math
 import shutil
 
 logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.INFO)
+logger = logging.getLogger(__name__)
 
 
 class ImgurFactory:
@@ -48,6 +49,8 @@ class ImgurFactory:
             return KDEImgur
         elif is_gui and sys.platform == 'darwin':
             return MacImgur
+        elif is_gui and os.environ.get('DESKTOP_SESSION') == 'gnome':
+            return ZenityImgur
         else:
             return CLIImgur
 
@@ -84,9 +87,9 @@ class Imgur():
             'https://api.imgur.com/oauth2/authorize?'
             'client_id={c_id}&response_type=pin&state=carlcarl'.format(c_id=self._client_id)
         )
-        self._gui_auth_msg = ('This is the first time you use this program, '
-                              'you have to visit this URL in your browser and copy the PIN code: \n')
-        self._cli_auth_msg = self._gui_auth_msg + self._auth_url
+        self._auth_msg = ('This is the first time you use this program, '
+                          'you have to visit this URL in your browser and copy the PIN code: \n')
+        self._auth_msg_with_url = self._auth_msg + self._auth_url
         self._token_msg = 'Enter PIN code displayed in the browser: '
         self._no_album_msg = 'Do not move to any album'
 
@@ -127,7 +130,7 @@ class Imgur():
                     if self.is_success(result):
                         return result['data']
                     else:
-                        logging.info('Reauthorize...')
+                        logger.info('Reauthorize...')
                         self.request_new_tokens_and_update()
                         self.write_tokens_to_config()
                         time.sleep(mdelay)
@@ -162,13 +165,13 @@ class Imgur():
         try:
             self._access_token = parser.get('Token', 'access_token')
         except:
-            logging.warning('Can\'t find access token, set to empty')
+            logger.warning('Can\'t find access token, set to empty')
             self._access_token = None
 
         try:
             self._refresh_token = parser.get('Token', 'refresh_token')
         except:
-            logging.warning('Can\'t find refresh token, set to empty')
+            logger.warning('Can\'t find refresh token, set to empty')
             self._refresh_token = None
 
     def _request(self, method, url, body, headers):
@@ -198,11 +201,11 @@ class Imgur():
                 # If without assigning a value to access_token,
                 # then just read the value from config file
                 self.set_tokens_using_config()
-            logging.info('Get album list with access token')
-            logging.debug('Access token: {token}'.format(token=self._access_token))
+            logger.info('Get album list with access token')
+            logger.debug('Access token: {token}'.format(token=self._access_token))
             headers = {'Authorization': 'Bearer {token}'.format(token=self._access_token)}
         else:
-            logging.info('Get album list without a access token')
+            logger.info('Get album list without a access token')
             headers = {'Authorization': 'Client-ID {c_id}'.format(c_id=self._client_id)}
 
         self._request('GET', url, None, headers)
@@ -292,8 +295,8 @@ class Imgur():
             True if success, else False
         '''
         if ('success' in response) and (not response['success']):
-            logging.info(response['data']['error'])
-            logging.debug(json.dumps(response))
+            logger.info(response['data']['error'])
+            logger.debug(json.dumps(response))
             return False
         return True
 
@@ -306,8 +309,8 @@ class Imgur():
             result: The result return from the server
             config: The name of the config file
         '''
-        logging.debug('Access token: %s', self._access_token)
-        logging.debug('Refresh token: %s', self._refresh_token)
+        logger.debug('Access token: %s', self._access_token)
+        logger.debug('Refresh token: %s', self._refresh_token)
 
         parser = SafeConfigParser()
         parser.read(self.CONFIG_PATH)
@@ -423,13 +426,13 @@ class Imgur():
                 albums = self.request_album_list()
                 album_id = self.ask_album_id(albums)
                 if album_id is not None:
-                    logging.info('Upload the image to the album...')
+                    logger.info('Upload the image to the album...')
                     data['album_id'] = album_id
                 else:
                     # If it's None, means user doesn't want to upload to any album
-                    logging.info('Upload the image...')
+                    logger.info('Upload the image...')
             else:
-                logging.info('Upload the image to the album...')
+                logger.info('Upload the image to the album...')
                 data['album_id'] = album_id
 
             files = {'image': image_path}
@@ -439,7 +442,7 @@ class Imgur():
         self._request('POST', url, body, headers)
         result = self._get_json_response()
         if not self.is_success(result):
-            logging.info('Reauthorize...')
+            logger.info('Reauthorize...')
             self.request_new_tokens_and_update()
             self.write_tokens_to_config()
             self._request('POST', url, body, headers)
@@ -452,11 +455,11 @@ class Imgur():
 class CLIImgur(Imgur):
 
     def show_error_and_exit(self, msg="Error"):
-        logging.error(msg)
+        logger.error(msg)
         sys.exit(1)
 
     def ask_pin(self):
-        print(self._cli_auth_msg)
+        print(self._auth_msg_with_url)
         pin = raw_input(self._token_msg)
         return pin
 
@@ -486,39 +489,42 @@ class CLIImgur(Imgur):
 class KDEImgur(Imgur):
 
     def show_error_and_exit(self, msg="Error"):
+        args = [
+            'kdialog',
+            '--error',
+            msg,
+        ]
         show_error_dialog = subprocess.Popen(
-            [
-                'kdialog',
-                '--error',
-                msg
-            ],
+            args,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
         show_error_dialog.communicate()
-        logging.error(msg)
+        logger.error(msg)
         sys.exit(1)
 
     def ask_pin(self):
+        args = [
+            'kdialog',
+            '--msgbox',
+            self._auth_msg_with_url,
+        ]
         auth_msg_dialog = subprocess.Popen(
-            [
-                'kdialog',
-                '--msgbox',
-                self._auth_msg
-            ],
+            args,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
         auth_msg_dialog.communicate()
 
+        args2 = [
+            'kdialog',
+            '--title',
+            'Input dialog',
+            '--inputbox',
+            self._token_msg,
+        ]
         ask_pin_dialog = subprocess.Popen(
-            [
-                'kdialog',
-                '--title',
-                'Input dialog',
-                '--inputbox',
-                self._token_msg
-            ],
+            args2,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
@@ -527,12 +533,13 @@ class KDEImgur(Imgur):
         return pin
 
     def ask_image_path(self):
+        args = [
+            'kdialog',
+            '--getopenfilename',
+            '.',
+        ]
         ask_image_path_dialog = subprocess.Popen(
-            [
-                'kdialog',
-                '--getopenfilename',
-                '.'
-            ],
+            args,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
@@ -575,49 +582,52 @@ class KDEImgur(Imgur):
 class MacImgur(Imgur):
 
     def show_error_and_exit(self, msg="Error"):
+        args = [
+            'osascript',
+            '-e',
+            (
+                'tell app "Finder" to display alert '
+                '"{msg}" as warning'.format(msg=msg)
+            ),
+        ]
         show_error_dialog = subprocess.Popen(
-            [
-                'osascript',
-                '-e',
-                (
-                    'tell app "Finder" to display alert '
-                    '"{msg}" as warning'.format(msg=msg)
-                )
-            ],
+            args,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
         show_error_dialog.communicate()
-        logging.error(msg)
+        logger.error(msg)
         sys.exit(1)
 
     def ask_pin(self):
+        args = [
+            'osascript',
+            '-e',
+            (
+                'tell app "SystemUIServer" to display dialog '
+                '"{msg}" default answer "{link}" '
+                'with icon 1'.format(msg=self._auth_msg, link=self._auth_url)
+            ),
+        ]
         auth_msg_dialog = subprocess.Popen(
-            [
-                'osascript',
-                '-e',
-                (
-                    'tell app "SystemUIServer" to display dialog '
-                    '"{msg}" default answer "{link}" '
-                    'with icon 1'.format(msg=self._auth_msg, link=self._auth_url)
-                )
-            ],
+            args,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
         auth_msg_dialog.communicate()
 
+        args2 = [
+            'osascript',
+            '-e',
+            (
+                'tell app "SystemUIServer" to display dialog '
+                '"{msg}" default answer "" with icon 1'.format(msg=self._token_msg)
+            ),
+            '-e',
+            'text returned of result',
+        ]
         ask_pin_dialog = subprocess.Popen(
-            [
-                'osascript',
-                '-e',
-                (
-                    'tell app "SystemUIServer" to display dialog '
-                    '"{msg}" default answer "" with icon 1'.format(msg=self._token_msg)
-                ),
-                '-e',
-                'text returned of result'
-            ],
+            args2,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
@@ -625,12 +635,13 @@ class MacImgur(Imgur):
         return pin
 
     def ask_image_path(self):
+        args = [
+            'osascript',
+            '-e',
+            'tell app "Finder" to POSIX path of (choose file with prompt "Choose Image:")',
+        ]
         ask_image_path_dialog = subprocess.Popen(
-            [
-                'osascript',
-                '-e',
-                'tell app "Finder" to POSIX path of (choose file with prompt "Choose Image:")'
-            ],
+            args,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
@@ -655,7 +666,7 @@ class MacImgur(Imgur):
                 'tell app "Finder" to choose from list '
                 '{{{l}}} with title "Choose From The List" with prompt "PickOne" '
                 'OK button name "Select" cancel button name "Quit"'.format(l=list_str[:-1])
-            )
+            ),
         ]
         choose_album_dialog = subprocess.Popen(arg, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         album_number = choose_album_dialog.communicate()[0].strip()
@@ -667,15 +678,18 @@ class MacImgur(Imgur):
 
     def show_link(self, result):
         link = result['data']['link'].replace('\\', '')
-        show_link_dialog = subprocess.Popen(
-            [
-                'osascript',
-                '-e',
+        args = [
+            'osascript',
+            '-e',
+            (
                 'tell app "Finder" to display dialog "Image Link" '
                 'default answer "{link}" '
                 'buttons {{"Show delete link", "OK"}} '
                 'default button 2'.format(link=link)
-            ],
+            ),
+        ]
+        show_link_dialog = subprocess.Popen(
+            args,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE
         )
@@ -683,19 +697,127 @@ class MacImgur(Imgur):
         response = response[response.rfind(':') + 1:]
         if response == 'Show delete link':
             delete_link = 'http://imgur.com/delete/{delete}'.format(delete=result['data']['deletehash'])
-            show_delete_link_dialog = subprocess.Popen(
-                [
-                    'osascript',
-                    '-e',
+            args2 = [
+                'osascript',
+                '-e',
+                (
                     'tell app "Finder" to display dialog "Delete link" '
                     'default answer "{link}"'.format(link=delete_link)
-                ],
+                ),
+            ]
+            show_delete_link_dialog = subprocess.Popen(
+                args2,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE
             )
             show_delete_link_dialog.communicate()
 
 
+class ZenityImgur(Imgur):
+
+    def show_error_and_exit(self, msg="Error"):
+        args = [
+            'zenity',
+            '--error',
+            '--text={text}'.format(text=msg),
+        ]
+        show_error_dialog = subprocess.Popen(
+            args,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE
+        )
+        show_error_dialog.communicate()
+        logger.error(msg)
+        sys.exit(1)
+
+    def ask_pin(self):
+
+        args = [
+            'zenity',
+            '--entry',
+            '--text={msg}'.format(msg=self._auth_msg),
+            '--entry-text={link}'.format(link=self._auth_url),
+        ]
+        auth_msg_dialog = subprocess.Popen(
+            args,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE
+        )
+        auth_msg_dialog.communicate()
+
+        args2 = [
+            'zenity',
+            '--entry',
+            '--text={msg}'.format(msg=self._token_msg),
+        ]
+        ask_pin_dialog = subprocess.Popen(
+            args2,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE
+        )
+        pin = ask_pin_dialog.communicate()[0].strip()
+        return pin
+
+    def ask_image_path(self):
+        args = [
+            'zenity',
+            '--file-selection',
+        ]
+        ask_image_path_dialog = subprocess.Popen(
+            args,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE
+        )
+        image_path = ask_image_path_dialog.communicate()[0].strip()
+        if image_path == '':  # Cancel dialog
+            sys.exit(1)
+
+        return image_path
+
+    def ask_album_id(self, albums):
+        i = 1
+        data_map = []
+        arg = [
+            'zenity',
+            '--list',
+            '--text="Choose the album"',
+            '--column=No.',
+            '--column=Album name',
+            '--column=Privacy',
+        ]
+        for album in albums:
+            arg.append(str(i))
+            arg.append('{album[title]}'.format(album=album))
+            arg.append('{album[privacy]}'.format(album=album))
+            data_map.append(album)
+            i += 1
+        arg.append(str(i))
+        arg.append(self._no_album_msg)
+        arg.append('public')
+        choose_album_dialog = subprocess.Popen(arg, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        album_number = choose_album_dialog.communicate()[0].strip()
+        if album_number == '':
+            self.show_error_and_exit('Album number should not be empty')
+        album_number = int(album_number)
+        return self._get_album_id(data_map, album_number)
+
+    def show_link(self, result):
+        link = 'Link: {link}'.format(link=result['data']['link'].replace('\\', ''))
+        links = (link + '\n' +
+                 'Delete link: http://imgur.com/delete/{delete}'.format(delete=result['data']['deletehash']))
+        args = [
+            'zenity',
+            '--info',
+            '--text={links}'.format(links=links),
+        ]
+        show_link_dialog = subprocess.Popen(
+            args,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE
+        )
+        show_link_dialog.communicate()
+
+
 def main():
     parser = argparse.ArgumentParser()
     parser.add_argument(
